#include "Mat2d.h"
#include <assert.h>


Mat2d::Mat2d() :
   data(0), m(0), n(0) {
      
}

Mat2d::Mat2d(int m, int n) :
   m(m), n(n) {
   assert(m > 0 && n > 0);
   data = new double[m * n];
}

Mat2d::Mat2d(const Mat2d& M) :
   data(new double[M.m * M.n]), m(M.m), n(M.n) {
   double *myIt = data;
   const double* endM = M.data + m * n;
   for(const double* itM = M.data; itM != endM; ++itM)
      *myIt++ = *itM;
}

Mat2d::~Mat2d() {
   delete[] data;
}

void Mat2d::resize(int m, int n) {
   assert(m > 0 && n > 0);
   if(this->m * this->n < m * n) { // case to allocate new data
      delete[] data;
      data = new double[m * n];
   }
   this->m = m;
   this->n = n;
}

int Mat2d::getM() const {
   return m;
}

int Mat2d::getN() const {
   return n;
}

void Mat2d::getMN(int &m, int &n) const {
   m = this->m;
   n = this->n;
}

Mat2d& Mat2d::fill(double value) {
   for(double *it = data; it != data + m * n; ++it) {
      *it = value;
   }
   return *this;
}

const double& Mat2d::getElem(int i, int j) const {
   assert(i < m && j < n);
   return data[i * n + j];
}

double & Mat2d::getElem(int i, int j) {
   assert(i < m && j < n);
   return data[i * n + j];
}

Mat2d& Mat2d::add(const Mat2d& M) {
   assert(m == M.m && n == M.n);
   const double* itM = M.data;
   for(double *it = data; it != data + m * n; ++it) {
      *it += *itM++;
   }
   return *this;
}

bool Mat2d::equal(const Mat2d& M) const {
   if(m != M.m || n != M.n)
      return false;

   const double* itM = M.data;
   for(double *it = data; it != data + m * n; ++it) {
      if(*it != *itM++) {
         return false;
      }
   }
   return true;
}



// additional operators
// TODO


// operators

const double& Mat2d::operator()(int i, int j) const {
   return getElem(i, j);
}

double& Mat2d::operator()(int i, int j) {
   return getElem(i, j);
}

Mat2d& Mat2d::operator+=(const Mat2d& M) {
   assert(n == M.n && m == M.m);
   const double* itM = M.data;   
   for(double* it = data; it != data + n * m; ++it)
      *it += *itM++;
   return *this;
}

Mat2d& Mat2d::operator+=(double d) {
   for(double* it = data; it != data + n * m; ++it)
      *it += d;
   return *this;
}

Mat2d& Mat2d::operator-=(const Mat2d& M) {
   assert(n == M.n && m == M.m);
   const double* itM = M.data;   
   for(double* it = data; it != data + n * m; ++it)
      *it -= *itM++;
   return *this;
}

Mat2d& Mat2d::operator-=(double d) {
   for(double* it = data; it != data + n * m; ++it)
      *it -= d;
   return *this;
}

// elementwise multiplication
Mat2d& Mat2d::operator*=(const Mat2d& M) {
   assert(n == M.n && m == M.m);
   const double* itM = M.data;   
   for(double* it = data; it != data + n * m; ++it)
      *it *= *itM++;
   return *this;
}

Mat2d& Mat2d::operator*=(double d) {
   for(double* it = data; it != data + n * m; ++it)
      *it *= d;
   return *this;
}


// TODO: implement operators /=


Mat2d Mat2d::operator+(const Mat2d& M) const {
   Mat2d tmp(*this);
   tmp += M;
   return tmp;
}

Mat2d Mat2d::operator+(double d) const {
   Mat2d tmp(*this);
   tmp += d;
   return tmp;
}

Mat2d Mat2d::operator-(const Mat2d& M) const {
   Mat2d tmp(*this);
   tmp -= M;
   return tmp;
}

Mat2d Mat2d::operator-(double d) const {
   Mat2d tmp(*this);
   tmp -= d;
   return tmp;
}

Mat2d Mat2d::operator*(const Mat2d& M) const {
   Mat2d tmp(*this);
   tmp *= M;
   return tmp;
}

Mat2d Mat2d::operator*(double d) const {
   Mat2d tmp(*this);
   tmp *= d;
   return tmp;
}


// TODO: implement operators /
